/*
 * @author Stefan Hepp
 * @date 16.08.2005
 * @version $Id: SimpleCalculateAction.java,v 1.7 2006-01-13 15:25:39 herbert Exp $
 */

package com.gentics.portalnode.sdk.calculator;

import com.gentics.api.portalnode.action.GenericPluggableAction;
import com.gentics.api.portalnode.action.PluggableActionException;
import com.gentics.api.portalnode.action.PluggableActionRequest;
import com.gentics.api.portalnode.action.PluggableActionResponse;

/**
 * A calculate-pluggable-action. It calculates the result of two values with an operand.
 */
public class SimpleCalculateAction extends GenericPluggableAction {

    /**
     * It gets two integer values as parameters 'value1' and 'value2' and
     * adds/subtracts/.. them depending on the operation given in the parameter 'operand'.
     * The result is stored into the parameter 'value', the complete expression is stored as string
     * into 'expression'.
     */
    public boolean processAction(PluggableActionRequest request, PluggableActionResponse response) throws PluggableActionException {

        int val1;
        int val2;
        int result = 0;

        // try to get the parameters and convert them into ints
        try {
            val1 = Integer.parseInt(request.getParameter("value1").toString());
            val2 = Integer.parseInt(request.getParameter("value2").toString());
        } catch (NumberFormatException e) {
            response.setFeedbackMessage("Invalid number.");
            return false;
        }

        // get the operand ..
        String operand = request.getParameter("operand").toString();

        try {
            // .. and calculate the result
            if ("+".equals(operand)) {
                result = val1 + val2;
            } else if ("-".equals(operand)) {
                result = val1 - val2;
            } else if ("*".equals(operand)) {
                result = val1 * val2;
            } else if ("/".equals(operand)) {
                result = val1 / val2;
            }
        } catch (ArithmeticException e) {
            response.setFeedbackMessage("Invalid operation.");
            return false;
        }

        // store the result and the complete expression in the response
        response.setParameter("value", Integer.toString(result));
        response.setParameter("expression", val1 + " " + operand + " " + val2 + " = " + result);

        return true;
    }

}
