/*
 * @author norbert
 * @date 17.03.2006
 * @version $Id: UserBean.java,v 1.3 2006-05-22 11:49:55 herbert Exp $
 */
package com.gentics.portalnode.sdk.resolvable;

import com.gentics.api.lib.resolving.ChangeableBean;

/**
 * Implementation of the user object. The class has the class-members
 * {@link #firstName}, {@link #lastName} and {@link #id}. The implementation
 * extends {@link com.gentics.api.lib.resolving.ChangeableBean}, so that it can
 * be directly used in a
 * {@link com.gentics.api.lib.datasource.ResolvableDatasource}. Additionally,
 * the method {@link #equals(Object)} is implemented to recognize two instances
 * of {@link UserBean} as equal, when they have the same {@link #id}.
 */
public class UserBean extends ChangeableBean {
    /**
     * Serial Version UID
     */
    private static final long serialVersionUID = -6565143193370351341L;

    /**
     * Unique id of the object
     */
    private String id;

    /**
     * first name
     */
    private String firstName;

    /**
     * last name
     */
    private String lastName;

    /**
     * Internal sequence to generate unique ids
     */
    private static int uniqueIdSequence = 0;

    /**
     * Create an instance of the user bean and set a new unique id
     */
    public UserBean() {
        // set a unique id
        id = getUniqueId();
    }

    /**
     * Internal method that generates unique id's
     * @return a unique id
     */
    private static synchronized String getUniqueId() {
        return Integer.toString(++uniqueIdSequence);
    }

    /**
     * Get the firstname
     * @return Returns the firstName.
     */
    public String getFirstName() {
        return firstName;
    }

    /**
     * Set the firstname
     * @param firstName The firstName to set.
     */
    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    /**
     * Get the object's id
     * @return Returns the id.
     */
    public String getId() {
        return id;
    }

    /**
     * Set the id
     * @param id The id to set.
     */
    public void setId(String id) {
        // prevent overwriting existing ids with empty strings
        if ((id != null && id.length() > 0) || this.id == null || this.id.length() == 0) {
            this.id = id;
        }
    }

    /**
     * Get the lastname
     * @return Returns the lastName.
     */
    public String getLastName() {
        return lastName;
    }

    /**
     * Set the lastname
     * @param lastName The lastName to set.
     */
    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (obj instanceof UserBean) {
            // two user objects are equal, when they have the same id
            UserBean otherUser = (UserBean) obj;
            return id.equals(otherUser.getId());
        } else {
            return super.equals(obj);
        }
    }
}
